#!/usr/bin/env python3

# This script pulls (in one hit) the data from
# Huntingdon District Council District Council Bins Data
from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page, **kwargs) -> None:
        # Make a BS4 object
        soup = BeautifulSoup(page.text, features="html.parser")
        soup.prettify()

        data = {"bins": []}

        bin_types = ["Domestic", "Recycle", "Organic"]

        for i, date in enumerate(soup.find("ul", class_="d-print-none").find_all("li")):
            data["bins"].append(
                {
                    "type": bin_types[i],
                    "collectionDate": date.find("strong").get_text(strip=True),
                }
            )
            ++i

        return data



from unittest import mock
import pytest
from uk_bin_collection.common import *
from io import StringIO
from contextlib import redirect_stdout
from unittest.mock import patch, MagicMock
from selenium import webdriver


def test_check_postcode_valid():
    valid_postcode = "SW1A 1AA"
    result = check_postcode(valid_postcode)
    assert result is True


def test_check_postcode_invalid():
    invalid_postcode = "BADPOSTCODE"
    with pytest.raises(ValueError) as exc_info:
        result = check_postcode(invalid_postcode)
    assert exc_info._excinfo[1].args[0] == "Exception: Invalid postcode Status: 404"
    assert exc_info.type == ValueError


def test_check_paon():
    valid_house_num = "1"
    result = check_paon(valid_house_num)
    assert result is True


def test_check_paon_invalid(capfd):
    invalid_house_num = None
    with pytest.raises(SystemExit) as exc_info:
        result = check_paon(invalid_house_num)
    out, err = capfd.readouterr()
    assert out.startswith("Exception encountered: Invalid house number")
    assert exc_info.type == SystemExit
    assert exc_info.value.code == 1


def test_get_data_check_uprn():
    uprn = "1"
    result = check_uprn(uprn)
    assert result is True


def test_get_data_check_uprn_exception(capfd):
    uprn = None
    result = check_uprn(uprn)
    out, err = capfd.readouterr()
    assert out.startswith("Exception encountered: ")


def test_get_data_check_usrn():
    usrn = "1"
    result = check_usrn(usrn)
    assert result is True


def test_get_data_check_usrn_exception(capfd):
    usrn = None
    result = check_usrn(usrn)
    out, err = capfd.readouterr()
    assert out.startswith("Exception encountered: ")


def test_get_date_with_ordinal():
    date_number = 1
    result = get_date_with_ordinal(date_number)
    assert result == "1st"


def test_get_date_with_ordinal_exception():
    date_number = "a"
    with pytest.raises(TypeError) as exc_info:
        result = get_date_with_ordinal(date_number)
    assert exc_info.type == TypeError
    assert (
            exc_info.value.args[0] == "not all arguments converted during string formatting"
    )


def test_parse_header():
    input_header = "i:am|:a:test:header|value:test"
    result = parse_header(input_header)
    assert result == {"i": "am", ":a": "test:header", "value": "test"}
    assert type(result) is dict


def test_is_holiday():
    date = "20"
    result = is_holiday("2022, 12, 25")
    assert result is True

def test_is_holiday():
    date = "20"
    result = is_holiday("2022, 12, 25")
    assert result is True

def test_is_holiday_region():
    date = "20"
    result = is_holiday("2022, 12, 25", Region.WLS)
    assert result is True


def test_remove_alpha_characters():
    test_string = "12345abc12345"
    result = remove_alpha_characters(test_string)
    assert result == "1234512345"


def test_remove_alpha_characters_bad():
    test_string = "12345abc12345"
    result = remove_alpha_characters(test_string)
    assert result != "12345abc12345"


def test_get_dates_every_x_days():
    now = datetime(2023, 2, 25, 7, 7, 17, 748661)
    result = get_dates_every_x_days(now, 5, 7)
    assert len(result) == 7
    assert result[6] == "27/03/2023"


def test_get_dates_every_x_days_bad():
    now = datetime(2023, 2, 25, 7, 7, 17, 748661)
    result = get_dates_every_x_days(now, 5, 7)
    assert len(result) != 8
    assert result[6] != "27/03/2022"


def test_remove_ordinal_indicator_from_date_string():
    test_string = "June 12th 2022"
    result = remove_ordinal_indicator_from_date_string(test_string)
    assert result == "June 12 2022"


def test_remove_ordinal_indicator_from_date_string_bad():
    test_string = "June 12th 2022"
    result = remove_ordinal_indicator_from_date_string(test_string)
    assert result != "June 12th 2022"


def test_get_weekday_dates_in_period():
    now = datetime(2023, 2, 25, 7, 7, 17, 748661)
    result = get_weekday_dates_in_period(now, 5, 7)
    assert len(result) == 7
    assert result[6] == "08/04/2023"


def test_get_weekday_dates_in_period_bad():
    now = datetime(2023, 2, 25, 7, 7, 17, 748661)
    result = get_weekday_dates_in_period(now, 5, 7)
    assert len(result) != 8
    assert result[6] != "08/04/20232"

def test_get_next_occurrence_from_day_month_false():
    result = get_next_occurrence_from_day_month(datetime(2023,12,1))
    assert result == datetime(2023, 12, 1, 0, 0)

def test_get_next_occurrence_from_day_month_true():
    result = get_next_occurrence_from_day_month(datetime(2023,9,1))
    assert result == pd.Timestamp('2024-09-01 00:00:00')

def test_update_input_json():
    council = "test_council"
    url = "TEST_URL"
    postcode="TEST_POSTCODE"
    uprn="TEST_UPRN"
    web_driver="TEST_WEBDRIVER"
    skip_get_url = True
    update_input_json(council, url, postcode=postcode, uprn=uprn, web_driver=web_driver, skip_get_url=skip_get_url)
    cwd = os.getcwd()
    input_file_path = os.path.join(cwd, "uk_bin_collection", "tests", "input.json")
    result1 = os.path.exists(input_file_path)     
    with open(input_file_path, 'r') as f:
        data = json.load(f)   
    assert result1 == True
    assert data[council] == {"postcode": postcode, "skip_get_url": skip_get_url, "uprn": uprn, "url": url, "web_driver": web_driver, "wiki_name": council}

def test_update_input_json_fail(capsys, monkeypatch):
    def mock_os_path_exists(path):
        return False  # Simulate the path not existing

    monkeypatch.setattr(os.path, 'exists', mock_os_path_exists)

    council = "test_council"
    url = "TEST_URL"
    postcode="TEST_POSTCODE"
    uprn="TEST_UPRN"
    web_driver="TEST_WEBDRIVER"
    skip_get_url = True
    update_input_json(council, url, postcode=postcode, uprn=uprn, web_driver=web_driver, skip_get_url=skip_get_url)

    captured = capsys.readouterr()
    assert "Exception encountered: Unable to update input.json file for the council." in captured.out
    assert "Please check you're running developer mode" in captured.out

def test_create_webdriver_local():
    result = create_webdriver(None)
    assert result.name == 'chrome'

def test_create_webdriver_remote():
    result = create_webdriver("http://selenium:4444")
    assert result.name == 'chrome'



from bs4 import BeautifulSoup
import time
from dateutil.relativedelta import relativedelta
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import Select
from selenium.common.exceptions import NoSuchElementException

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
     AbstractGetBinDataClass

class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        page = "https://www.midandeastantrim.gov.uk/resident/waste-recycling/collection-dates/"

        # Assign user info
        user_postcode = kwargs.get("postcode")
        # not used: user_paon = kwargs.get("paon")
        web_driver = kwargs.get("web_driver")

        # Create Selenium webdriver
        options = webdriver.ChromeOptions()
        options.add_experimental_option('excludeSwitches', ['enable-logging'])
        driver = create_webdriver(web_driver)
        
        driver.get(page)

        time.sleep(5)
        number=0
        driver.switch_to.frame(number)
        # Enter postcode in text box and wait
        inputElement_pc = driver.find_element(
            By.ID, "txtAjaxSearch"
        )
        inputElement_pc.send_keys(user_postcode)

        time.sleep(5)

        # Submit address information and wait - selecting the top one only
        # if it is an exact match then it will go straight to the results
        try:
            button = driver.find_element(
                By.XPATH, '//*[@id="show-button-0"]'
            )
            driver.execute_script("arguments[0].click();", button)
        except NoSuchElementException:
            pass

        time.sleep(4)

        # Read next collection information
        page = driver.find_element(
            By.ID, "divCalendarGraphics"
        ).get_attribute("outerHTML")
        
        # Make a BS4 object - remove bold tags and add @ so we can split the lines later
        soup = BeautifulSoup(page.strip().replace("<b>", "").replace("</b>", "").replace("<br>", "@"), features="html.parser")
        soup.prettify()

        # Data to return
        data = {"bins": []}

        # Valid bin types
        binTypes = [
            "Refuse",
            "Garden"
        ]

        # Value to create dict for bin values
        keys, values = [], []

        # Loop though html for text containing bins
        # example of html (bold tags removed above)
        # <div id="divCalendarGraphics">
        # <br>  <b>Refuse</b>: Tue 14 Nov then every alternate  Tue<br><b>Recycling</b>: No Recycling waste collection for this address<br><b>Garden</b>: Tue 21 Nov then every alternate  Tue<br><img src="img/Gif-Spacer.gif" alt="spacer" height="1" width="30">
        # split by br tag and take first 4 splits
        lines = soup.text.split('@',4)
        for line in lines[1:4]:
            keys.append(line.split(':')[0].strip())
            # strip out the day and month from the text
            values.append(line.split(':')[1].strip().split(' ')[:3])

        # Create dict for bin name and string dates
        binDict = dict(zip(keys, values))

        # Process dict for valid bin types
        for bin in list(binDict):
            if bin in binTypes:
                # Convert date - no year value so take it from todays date
                if binDict[bin][0] == "Tomorrow":
                    date = datetime.today() + relativedelta(days=1)
                elif binDict[bin][0] == "Today":
                    date = datetime.today()
                else:
                    date = datetime.strptime(' '.join(binDict[bin][1:]), "%d %b").replace(year=datetime.today().year)
                    # if the date is in the past then it means the collection is next year so add a year
                    if date < datetime.today():
                        date = date + relativedelta(years=1)

                # Set bin data
                dict_data = {
                    "type": bin,
                    "collectionDate": date.strftime(date_format),
                }
                data["bins"].append(dict_data)

        # Quit Selenium webdriver to release session
        driver.quit()

        return data



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        api_url = "https://swict.malvernhills.gov.uk/mhdcroundlookup/HandleSearchScreen"

        user_uprn = kwargs.get("uprn")
        # Check the UPRN is valid
        check_uprn(user_uprn)

        # Create the form data
        form_data = {"nmalAddrtxt": "", "alAddrsel": user_uprn}
        # expects postcode to be looked up and then uprn used.
        # we can just provide uprn

        # Make a request to the API
        requests.packages.urllib3.disable_warnings()
        response = requests.post(api_url, data=form_data, verify=False)

        # Make a BS4 object
        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()

        # Find results table
        table_element = soup.find("table")
        table_body = table_element.find("tbody")
        rows = table_body.find_all("tr")

        data = {"bins": []}

        for row in rows:
            columns = row.find_all("td")
            columns = [ele.text.strip() for ele in columns]

            thisCollection = [ele for ele in columns if ele]  # Get rid of empty values

            # if not signed up for garden waste, this appears as Not applicable
            if "Not applicable" not in thisCollection[1]:
                bin_type = thisCollection[0].replace("collection", "").strip()
                date = datetime.strptime(thisCollection[1], "%A %d/%m/%Y")
                dict_data = {
                    "type": bin_type,
                    "collectionDate": date.strftime(date_format),
                }
                data["bins"].append(dict_data)

        return data
